/*-------------------------------------------------------------------
Name......... cctalkcoin.h
Author....... Lee Weekes
Date......... 29/04/07
Description.. Header file for cctalk SR5i coin driver.
-------------------------------------------------------------------*/
/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/
#ifndef PERIPHERAL_SR5I
#define PERIPHERAL_SR5I

#define TOTAL_COINS					16	/* Total number of coins accepted	*/

enum
{
	COIN_EVENT_POLL,
	COIN_EVENT_STATUS,
	COIN_EVENT_CREDITS,
	COIN_EVENT_INHIBIT,
	COIN_EVENT_SORTERPATH,
	COIN_EVENT_OTHER,
	COIN_EVENT_ERROR,
	COIN_TOTAL_EVENTS
};

#define MAX_POLLING_QUEUE			0x09

#define POLL_CHANGE					0x01
#define CREDIT_CHANGE				0x02
#define STATUS_CHANGE				0x04
#define INHIBIT_CHANGE				0x08
#define SORTERPATH_CHANGE			0x10
#define SORTEROVERRIDE_CHANGE		0x20
#define MASTERINHIBIT_CHANGE		0x40
#define ERROR_CHANGE				0x80
#define BUS_BUSY					0x01
#define TIMEOUT_LENGTH				0x03

#define FLIGHTDECK_OPEN				0xFE
#define NO_DEVICE					0xFF

#define STANDARD_POLL				0x00
#define CREDIT_POLL					0x01

#define HOSTADDRESS					0x01

#define MAXIMUM_EVENTS				0x05

#define COIN_DEFAULT_POLLINGRATE	200
#define COIN_MAXIMUM_COINS			0xF
#define COIN_MAXIMUM_EVENTS			32
#define MAXIMUM_POLLING_STATES		0x06
#define MAXIMUM_NUMBER_EVENTS		0x05

#define CCTALK_COIN_NEXTSET			2
#define CCTALK_PRIORITYSTATES		22
typedef	struct cctalkcoincfg
{
	UBYTE slaveport;
	UBYTE slaveaddress;
	const SECURITY_KEY *thekey;
} CCTALKCOINCFG;

typedef struct
{

	UWORD counts;
	WORD checksum;

} COIN_VALUE;

typedef struct
{
	UBYTE 		eventnumber;				/* the event number 	*/
	COIN_VALUE 	coincounts[TOTAL_COINS];	/* the coin values 		*/
} COIN_DATA;

typedef struct coinDevice
{
	UBYTE			run;			/* Do we poll or not			*/
	UBYTE 			port;
	UBYTE 			slaveaddress;
	UBYTE 			hostaddress;
	UBYTE 			*channels;
	UBYTE 			detected;
	UBYTE 			enabled;

	UBYTE 			smachine;		/* Status + internal state 		*/
	UBYTE 			nextstate;
	UBYTE 			status;

	UWORD 			pollingrate;	/* The priority 				*/
	UBYTE 			polltype;		/* Switch basic poll and credit */

	ULONG 			credits;		/* The credits					*/

	UBYTE 			faultcode;		/* Perform self-test variables 	*/
	UBYTE 			faultinfo;

	UBYTE 			coinposition;	/* The coin position for sorter */

	UWORD 			inhibits;
	UWORD 			oldinhibits;
	UWORD 			setinhibits;

	ULONG 			sorterpaths;
	UBYTE 			sorteroverride;

	UBYTE			inerror;

	UBYTE			sorterpathscoin;
	UBYTE			creditpollattempt;
	UBYTE			lpactive;
	UBYTE			maintenanceroutine;

	UBYTE			selftest;

	ULONG 			paths[TOTAL_COINS];
	UBYTE			cprocessing;
	UWORD 			lasterror;
	ULONG 			sevents;		/* The event code of the device	*/
	SECURITY_KEY 	key;			/* The SECURITY_KEY	with CRC+Key*/

	TIMER2 			polltimer;		/* The polling timer			*/
} COINDEVICE;

#define LP_SET_INHIBITS			0x01
#define LP_COIN_PROCESSING		0x02
#define LP_RESET				0x04
#define LP_SELF_TEST			0x08
#define LP_SORTER_OVERRIDE		0x10
#define LP_SORTER_PATHS			0x20

/* Error event codes				*/
#define LOST_EVENT				1
#define LOST_DEVICE				2

typedef	struct
{
								/* Enable the coin processing 	 	*/
	void  (*enable_coin_processing)(const DEVICE* dev);

								/* Disable the coin processing 	 	*/
	void  (*disable_coin_processing)(const DEVICE* dev);

								/* Get the sorter paths			 	*/
	void (*get_sorter_paths)(const DEVICE *dev, UBYTE coinPosition, ULONG *paths);

								/* Set the sorter paths			 	*/
	void  (*set_sorter_paths)(const DEVICE* dev, UBYTE coinPosition, ULONG paths);

								/* Set the sorter overrides		 	*/
	void  (*set_sorter_override)(const DEVICE* dev, UBYTE override);

								/* Obtain the override settings	 	*/
	UBYTE (*get_sorter_override)(const DEVICE* dev);

								/* Enable all coins				 	*/
	void  (*enable_all_coins)(const DEVICE* dev);

								/* Disable all coins			 	*/
	void  (*disable_all_coins)(const DEVICE* dev);

								/* Enable a denomination			*/
	void  (*enable_coin)(const DEVICE* dev, UBYTE coin);

								/* Disable a denomination			*/
	void  (*disable_coin)(const DEVICE *dev, UBYTE coin);

								/* Obtain bitmask of enabled coins	*/
	UWORD (*get_all_enabled_coins)(const DEVICE *dev);

								/* Specific denomination enabled?	*/
	BYTE  (*is_coin_enabled)(const DEVICE *dev, UBYTE coin);

								/* Return the status of the device	*/
	UBYTE (*get_coin_status)(const DEVICE *dev);

								/* Perform a self test 				*/
	void  (*selftest)(const DEVICE *dev);

								/* Perform a reset of the device	*/
	void  (*reset)(const DEVICE *dev);

								/* Set the polling rate 			*/
	void  (*set_polling_rate)(const DEVICE *dev, UBYTE pollingRate);

								/* Any events pending?				*/
	UBYTE (*get_coin_events)(const DEVICE *dev);

								/* Returns up to last 4 errors		*/
	UBYTE  (*get_coin_errors)(const DEVICE *dev);

								/* Number of coins for denomination	*/
	UWORD (*get_coin_count)(const DEVICE *dev, UBYTE coin);

								/* Subtract 1 from the denomination	*/
	BYTE  (*subtract_from_coin_counts)(const DEVICE *dev, UBYTE coin);

								/* Add 1 to coin count				*/
	BYTE  (*add_to_coin_counts)(const DEVICE *dev, UBYTE coin);

} COINSERVICES;

/* Defines for the state machine functions. The actions are paired together
   therefore on incrementing a transmit state you can obtain the receive.

   The first 4 pairs are executed in the 10ms interrupt						*/
enum
{
	CCTALK_COIN_SIMPLEPOLL,
	CCTALK_COIN_RECEIVESIMPLEPOLL,

	CCTALK_COIN_GETCREDITS,
	CCTALK_COIN_RECEIVEGETCREDITS,

	CCTALK_COIN_TRANSMITSTATUS,
	CCTALK_COIN_RECEIVESTATUS,

	CCTALK_COIN_TRANSMITGETINHIBITS,
	CCTALK_COIN_RECEIVEGETINHIBITS,

	CCTALK_COIN_TRANSMITLOWPRIORITY,
	CCTALK_COIN_RECEIVELOWPRIORITY,

	CCTALK_COIN_MODIFYINHIBITS,
	CCTALK_COIN_RECEIVEMODIFYINHIBITS,

	CCTALK_COIN_ENABLE,
	CCTALK_COIN_RECEIVEENABLE,

	CCTALK_COIN_RESET,
	CCTALK_COIN_RECEIVERESET,

	CCTALK_COIN_PERFORMTEST,
	CCTALK_COIN_RECEIVEPERFORMTEST,

	CCTALK_COIN_TRANSMITSETSORTEROVERRIDE,
	CCTALK_COIN_RECEIVESETSORTEROVERRIDE,

	CCTALK_COIN_TRANSMITSETSORTERPATHS,
	CCTALK_COIN_RECEIVESETSORTERPATHS,


	CCTALK_COIN_TRANSMITGETSORTEROVERRIDE,
	CCTALK_COIN_RECEIVEGETSORTEROVERRIDE,

	CCTALK_COIN_TRANSMITGETSORTERPATHS,
	CCTALK_COIN_RECEIVEGETSORTERPATHS,


	CCTALK_COIN_IDLE,
	CCTALK_COIN_ERROR,
	CCTALK_COIN_TOTAL_STATES
};

#include "cctalkcoin.p"

#endif